

#include "ADC.h"
#include "PWM.h"
#include "main.h"
#include "Funktionen.h"

void Init_ADC ( void )
{
	/* Set clock */
	RCC_AHBPeriphClockCmd( RCC_AHBPeriph_GPIOA, ENABLE );
	RCC_AHBPeriphClockCmd( RCC_AHBPeriph_DMA1 , ENABLE );
	RCC_APB2PeriphClockCmd( RCC_APB2Periph_ADC1, ENABLE );
	RCC_ADCCLKConfig( RCC_ADCCLK_HSI14 );


	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_StructInit( &GPIO_InitStructure );

	ADC_InitTypeDef ADC_InitStructure;
	ADC_StructInit( &ADC_InitStructure );

	DMA_InitTypeDef DMA_InitStructure;
	DMA_StructInit( &DMA_InitStructure );

	TIM_TimeBaseInitTypeDef TIM_TimeBase_InitStructure;
	TIM_TimeBaseStructInit( &TIM_TimeBase_InitStructure );

	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_Init( &NVIC_InitStructure );


	/* ADC Kanle */
	/* IO initialisieren */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init( GPIOA, &GPIO_InitStructure );

	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 14;
	NVIC_Init( &NVIC_InitStructure );

	/* DMA Control */
	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&(ADC1->DR);
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)DataBuffer;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure.DMA_BufferSize = NR_OF_VALUES;
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_Init( DMA1_Channel1, &DMA_InitStructure );
	DMA_ITConfig( DMA1_Channel1,DMA_IT_TC,ENABLE );

	/* ADC */
	ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_Rising;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T1_TRGO;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_ScanDirection = ADC_ScanDirection_Upward;
	ADC_Init( ADC1, &ADC_InitStructure );

	ADC_ChannelConfig( 	ADC1,
						ADC_Channel_0 |
						ADC_Channel_2 |
						ADC_Channel_3 |
						ADC_Channel_4 |
						ADC_Channel_5 |
						ADC_Channel_6
						,ADC_SampleTime_1_5Cycles );
	ADC_GetCalibrationFactor( ADC1 );

	DMA_Cmd( DMA1_Channel1, ENABLE );
	ADC_Cmd( ADC1, ENABLE );
	ADC_DMACmd( ADC1, ENABLE );
	ADC_DMARequestModeConfig( ADC1,ADC_DMAMode_Circular );
	while(ADC_GetFlagStatus( ADC1, ADC_FLAG_ADEN ) == RESET );
	ADC_StartOfConversion( ADC1 );
}


void DMA1_Channel1_IRQHandler ( void )
{
	uint16_t Bemf = 0;
	static uint32_t rpmCount[NR_OF_RPM_MIDDLE];
	static uint16_t Index = 0;
	uint32_t Buffer = 0;
	static uint32_t VoltageBuffer = 0;

	rpmCount[Index]++;

	// Transver complete
	DMA_ClearITPendingBit( DMA1_IT_GL1 );

	// Stop PWM @ Overcurrent for min. 1 Cycle
	if (DataBuffer[I_DC] > CURRENTLIMIT && Motorstate == ON && !Flags.RunUp ) {
		Flags.Overcurrent = CURRENTLIMITTIME;
		Motorstate = OFF;
	}
	Dummy = Middle_Current ( DataBuffer[I_DC] ) ;

	if ( Flags.Overcurrent ) {
		TIM1->EGR |= TIM_EGR_BG;
	}else{
		TIM1->BDTR |= TIM_BDTR_MOE;
	}

	// Voltage to High
//	if ( DataBuffer[U_DC] < UNDERVOLTAGE || DataBuffer[U_DC] > OVERVOLTAGE ){
//		Flags.Voltagerange = ON;
//	}else{
//		Flags.Voltagerange = OFF;
//	}

	switch ( CommutationVektor ){
		case 0: Bemf = DataBuffer[BEMF2]; break;
		case 1: Bemf = DataBuffer[BEMF2]; break;
		case 2: Bemf = DataBuffer[BEMF1]; break;
		case 3: Bemf = DataBuffer[BEMF3]; break;
		case 4: Bemf = DataBuffer[BEMF2]; break;
		case 5: Bemf = DataBuffer[BEMF1]; break;
		case 6: Bemf = DataBuffer[BEMF3]; break;
		default: break;
	}

	// Get Fluxthreshold
	FluxThreshold = DataBuffer[RPM_SOLL_RAW]/2;

	if ( Motorstate == GET_POSITION ){
		static uint32_t Count = 0xffffffff;
		static uint32_t Meas1, Meas2, Meas3;

		Count++;
		if ( Count < 10 ){
			PREPARE_ADJUST();
			TIM1->CCR2 = 300;
		}

		else if ( Count < 20000 ){
			ADJUST();
		}

		else if ( Count < 20006 ){
			CommutationVektor = 1;
			NextCommutation( CommutationVektor );
			Meas1 += Bemf;
		}

		else if ( Count < 20012 ){
			CommutationVektor = 3;
			NextCommutation( CommutationVektor );
			Meas2 += Bemf;
		}

		else if ( Count < 20018 ){
			CommutationVektor = 5;
			NextCommutation( CommutationVektor );
			Meas3 += Bemf;
		}
		else {
			Meas1/=6;
			Meas2/=6;
			Meas3/=6;

			if ( Meas1 > Meas2 && Meas1 > Meas3 ) CommutationVektor = 1;
			if ( Meas2 > Meas1 && Meas2 > Meas3 ) CommutationVektor = 3;
			if ( Meas3 > Meas1 && Meas3 > Meas2 ) CommutationVektor = 5;

			Meas1 = 0;
			Meas2 = 0;
			Meas3 = 0;
			Count = 0xffffffff;

			if ( CommutationVektor != 1 ){
				Motorstate = GET_POSITION;
			}else{
				TIM1->CCR2 = PWM_MIN;
				Motorstate = ON;
				Flags.RunUp = RUNUPTIME;
			}
		}

	}

	else if ( Motorstate == ON ){
		if ( CommutationVektor % 2 == 1){
			if ( Bemf > (VoltageBuffer/2 + ((VoltageBuffer/2) * 30)/100) ){ //2200
				Fluxintegral += Bemf - DataBuffer[U_DC]/2;
			}
		}else{
			if ( Bemf < (VoltageBuffer/2 - ((VoltageBuffer/2) * 30)/100) ){ // 1400
				Fluxintegral += DataBuffer[U_DC]/2 - Bemf;
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////
		if ( (FluxThreshold < Fluxintegral || Flags.ForcecommutationTimer == 0) ){ // Commutate!
			if ( ++CommutationVektor >= 7 ){
				CommutationVektor = 1;
				if (++Index >= NR_OF_RPM_MIDDLE ) Index = 0;

				for ( uint8_t i = 0; i < NR_OF_RPM_MIDDLE; i++ ){
					Buffer += rpmCount[i];
				}
				Buffer /= NR_OF_RPM_MIDDLE;
				if (Buffer) rpm = (20000 * 60) / (Buffer * 7);
				Buffer = 0;
				rpmCount[Index] = 1;
			}

			NextCommutation( CommutationVektor );
			Fluxintegral = 0;
		}
	}else if ( Motorstate == OFF ){ // Clear all values if Motor is off
		Fluxintegral = 0;
		for ( uint8_t i = 0; i < NR_OF_RPM_MIDDLE; i++ ){
			rpmCount[i] = 1;
			rpm = 0;
		}
		POWERSTAGE_OFF();
		VoltageBuffer = DataBuffer[U_DC];
	}
}


